--
-- Full Stop
--
-- # Author:  Naduweisstschon
-- # date: 06.04.2019
--

FullStop = {}

function FullStop.prerequisitesPresent(specializations)
  return SpecializationUtil.hasSpecialization(Combine, specializations)
    and SpecializationUtil.hasSpecialization(FillUnit, specializations)
end

function FullStop.registerEventListeners(vehicleType)
  SpecializationUtil.registerEventListener(vehicleType, "onUpdate", FullStop)
end

function FullStop:onUpdate(dt)
  -- Only execute server-side
  if self.isServer then
    local hasPipe = false
	for vehicleType, vehicle in pairs(g_vehicleTypeManager.types) do
	  if vehicle ~= nil then
	  for name in pairs(vehicle.specializationsByName) do
	  	if name == "pipe" then
	  	hasPipe = true
	  	end
	  end
	  end
    end

    local spec = self.spec_drivable
    local hasPipe = self.spec_pipe
    -- local fillLevel = self:getFillUnitFillLevelPercentage(self:getCurrentDischargeNode().fillUnitIndex, 1)
    local fillLevel = self.spec_fillUnit:getFillUnitFillLevel(1)
    local fillCapacity = self.spec_fillUnit:getFillUnitCapacity(1)

    -- Combine is stopped
    if spec.stopped then
      
      -- if fillLevel < 1 then
	if fillLevel ~= fillCapacity then
        spec.stopped = false
      end

      if self:getLastSpeed() <= 0.1 then
        -- Hold the brakes for a little longer, in case the driver
        -- is holding the gas pedal and hasn't noticed, that the
        -- combine is full.
        spec.stopTime = spec.stopTime + dt
      end

      if spec.stopTime < 2000 then
        self:setBrakeLightsVisibility(true)
      
        -- The general brake force of the vehicle or something like that
        local brakeForce = self:getBrakeForce()

        -- Iterate over all the wheels of the vehicle
        for k, wheel in pairs(self:getWheels()) do
          -- Apply some magic forces
          setWheelShapeProps(wheel.node, wheel.wheelShape, 0, brakeForce * wheel.brakeFactor, wheel.steeringAngle, wheel.rotationDamping)
          setWheelShapeAutoHoldBrakeForce(wheel.node, wheel.wheelShape, brakeForce * wheel.autoHoldBrakeFactor)
        end

        -- Brake for good measure!??
        self:brake(brakeForce)
      end

      return
    end

    -- My mental health suffers from modding FS19
    if not spec.stopped and fillLevel == fillCapacity then
      self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_OFF)
      spec.stopped = true
      spec.stopTime = 0
	  if hasPipe then
      self:setPipeState(2)
	  end
    end

  end
end
